import { NetworkInterface } from './networkInterface';
import { Store } from './store';
import { Store as ReduxStore } from 'redux';
import { SelectionSet, GraphQLError } from 'graphql';
export declare class QueryManager {
    private networkInterface;
    private store;
    private selectionSetMap;
    private resultCallbacks;
    private idCounter;
    constructor({networkInterface, store}: {
        networkInterface: NetworkInterface;
        store: ReduxStore;
    });
    mutate({mutation, variables}: {
        mutation: string;
        variables?: Object;
    }): Promise<any>;
    watchQuery({query, variables, forceFetch, returnPartialData}: WatchQueryOptions): WatchedQueryHandle;
    broadcastNewStore(store: Store): void;
    watchSelectionSet(selectionSetWithRoot: SelectionSetWithRoot): WatchedQueryHandle;
    private stopQuery(queryId);
    private broadcastQueryChange(queryId, result);
    private handleQueryErrorsAndStop(queryId, errors);
    private registerResultCallback(queryId, callback);
    private makeRequest(queryId, request, selectionSet);
    private diffRequestAgainstStore(request, selectionSet);
}
export interface SelectionSetWithRoot {
    rootId: string;
    typeName: string;
    selectionSet: SelectionSet;
    variables: Object;
}
export interface WatchedQueryHandle {
    id: string;
    isStopped: () => boolean;
    stop(): any;
    onResult(callback: QueryResultCallback): any;
}
export declare type QueryResultCallback = (error: GraphQLError[], result?: any) => void;
export interface WatchQueryOptions {
    query: string;
    variables?: Object;
    forceFetch?: boolean;
    returnPartialData?: boolean;
}
