"use strict";
var lodash_1 = require('lodash');
var parser_1 = require('./parser');
var storeUtils_1 = require('./storeUtils');
function writeFragmentToStore(_a) {
    var result = _a.result, fragment = _a.fragment, _b = _a.store, store = _b === void 0 ? {} : _b, variables = _a.variables;
    if (!fragment) {
        throw new Error('Must pass fragment.');
    }
    var parsedFragment = parser_1.parseFragment(fragment);
    var selectionSet = parsedFragment.selectionSet;
    return writeSelectionSetToStore({
        result: result,
        selectionSet: selectionSet,
        store: store,
        variables: variables,
    });
}
exports.writeFragmentToStore = writeFragmentToStore;
function writeQueryToStore(_a) {
    var result = _a.result, query = _a.query, _b = _a.store, store = _b === void 0 ? {} : _b, variables = _a.variables;
    var queryDefinition = parser_1.parseQuery(query);
    var resultWithDataId = lodash_1.assign({
        __data_id: 'ROOT_QUERY',
    }, result);
    return writeSelectionSetToStore({
        result: resultWithDataId,
        selectionSet: queryDefinition.selectionSet,
        store: store,
        variables: variables,
    });
}
exports.writeQueryToStore = writeQueryToStore;
function writeSelectionSetToStore(_a) {
    var result = _a.result, selectionSet = _a.selectionSet, _b = _a.store, store = _b === void 0 ? {} : _b, variables = _a.variables;
    if (!lodash_1.isString(result.id) && !lodash_1.isString(result.__data_id)) {
        throw new Error('Result passed to writeSelectionSetToStore must have a string ID');
    }
    var dataId = result['__data_id'] || result.id;
    selectionSet.selections.forEach(function (selection) {
        if (isField(selection)) {
            var resultFieldKey = storeUtils_1.resultKeyNameFromField(selection);
            var value = result[resultFieldKey];
            if (lodash_1.isUndefined(value)) {
                throw new Error("Can't find field " + resultFieldKey + " on result object " + dataId + ".");
            }
            writeFieldToStore({
                dataId: dataId,
                value: value,
                variables: variables,
                store: store,
                field: selection,
            });
        }
        else if (isInlineFragment(selection)) {
            writeSelectionSetToStore({
                result: result,
                selectionSet: selection.selectionSet,
                store: store,
                variables: variables,
            });
        }
        else {
            throw new Error('Non-inline fragments not supported.');
        }
    });
    return store;
}
exports.writeSelectionSetToStore = writeSelectionSetToStore;
function writeFieldToStore(_a) {
    var field = _a.field, value = _a.value, variables = _a.variables, store = _a.store, dataId = _a.dataId;
    var storeValue;
    var storeFieldName = storeUtils_1.storeKeyNameFromField(field, variables);
    if (lodash_1.isString(value) || lodash_1.isNumber(value) || lodash_1.isBoolean(value) || lodash_1.isNull(value)) {
        storeValue = value;
    }
    else if (lodash_1.isArray(value)) {
        if (lodash_1.isNull(field.selectionSet)) {
            storeValue = value;
        }
        else {
            var thisIdList_1 = [];
            value.forEach(function (item, index) {
                if (lodash_1.isNull(item)) {
                    thisIdList_1.push(null);
                }
                else {
                    var clonedItem = lodash_1.assign({}, item);
                    if (!lodash_1.isString(clonedItem.id)) {
                        clonedItem['__data_id'] = dataId + "." + storeFieldName + "." + index;
                    }
                    else {
                        clonedItem['__data_id'] = clonedItem.id;
                    }
                    thisIdList_1.push(clonedItem['__data_id']);
                    writeSelectionSetToStore({
                        result: clonedItem,
                        store: store,
                        selectionSet: field.selectionSet,
                        variables: variables,
                    });
                }
            });
            storeValue = thisIdList_1;
        }
    }
    else {
        var clonedValue = lodash_1.assign({}, value);
        if (!lodash_1.isString(clonedValue.id)) {
            clonedValue['__data_id'] = dataId + "." + storeFieldName;
        }
        else {
            clonedValue['__data_id'] = clonedValue.id;
        }
        writeSelectionSetToStore({
            result: clonedValue,
            store: store,
            selectionSet: field.selectionSet,
            variables: variables,
        });
        storeValue = clonedValue['__data_id'];
    }
    var newStoreObj = lodash_1.assign({}, store[dataId], (_b = {},
        _b[storeFieldName] = storeValue,
        _b
    ));
    store[dataId] = newStoreObj;
    var _b;
}
function isField(selection) {
    return selection.kind === 'Field';
}
function isInlineFragment(selection) {
    return selection.kind === 'InlineFragment';
}
//# sourceMappingURL=writeToStore.js.map