"use strict";
var chai_1 = require('chai');
var writeToStore_1 = require('../src/writeToStore');
var readFromStore_1 = require('../src/readFromStore');
describe('roundtrip', function () {
    it('real graphql result', function () {
        storeRoundtrip("\n      {\n        people_one(id: \"1\") {\n          name\n        }\n      }\n    ", {
            people_one: {
                name: 'Luke Skywalker',
            },
        });
    });
    it('with an alias', function () {
        storeRoundtrip("\n      {\n        luke: people_one(id: \"1\") {\n          name,\n        },\n        vader: people_one(id: \"4\") {\n          name,\n        }\n      }\n    ", {
            luke: {
                name: 'Luke Skywalker',
            },
            vader: {
                name: 'Darth Vader',
            },
        });
    });
    it('with variables', function () {
        storeRoundtrip("\n      {\n        luke: people_one(id: $lukeId) {\n          name,\n        },\n        vader: people_one(id: $vaderId) {\n          name,\n        }\n      }\n    ", {
            luke: {
                name: 'Luke Skywalker',
            },
            vader: {
                name: 'Darth Vader',
            },
        }, {
            lukeId: '1',
            vaderId: '4',
        });
    });
});
function storeRoundtrip(query, result, variables) {
    if (variables === void 0) { variables = {}; }
    var store = writeToStore_1.writeQueryToStore({
        result: result,
        query: query,
        variables: variables,
    });
    var reconstructedResult = readFromStore_1.readQueryFromStore({
        store: store,
        query: query,
        variables: variables,
    });
    chai_1.assert.deepEqual(result, reconstructedResult);
}
//# sourceMappingURL=roundtrip.js.map